@@ -5,14 +5,9 @@
  * Copyright (C) 2000-2003, Ximian, Inc.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <string.h>
 
 #include "soup-message-body.h"
-#include "soup.h"
 
 /**
  * SECTION:soup-message-body
@@ -83,7 +78,7 @@ typedef struct {
 /**
  * soup_buffer_new:
  * @use: how @data is to be used by the buffer
- * @data: (array length=length) (element-type guint8): data
+ * @data: data
  * @length: length of @data
  *
  * Creates a new #SoupBuffer containing @length bytes from @data.
@@ -114,7 +109,7 @@ soup_buffer_new (SoupMemoryUse use, gconstpointer data, gsize length)
 }
 
 /**
- * soup_buffer_new_take: (rename-to soup_buffer_new)
+ * soup_buffer_new_take:
  * @data: (array length=length) (transfer full): data
  * @length: length of @data
  *
@@ -127,6 +122,7 @@ soup_buffer_new (SoupMemoryUse use, gconstpointer data, gsize length)
  * Return value: the new #SoupBuffer.
  *
  * Since: 2.32
+ * Rename to: soup_buffer_new
  **/
 SoupBuffer *
 soup_buffer_new_take (guchar *data, gsize length)
@@ -170,7 +166,7 @@ soup_buffer_new_subbuffer (SoupBuffer *parent, gsize offset, gsize length)
 
 /**
  * soup_buffer_new_with_owner:
- * @data: (array length=length) (element-type guint8): data
+ * @data: data
  * @length: length of @data
  * @owner: pointer to an object that owns @data
  * @owner_dnotify: (allow-none): a function to free/unref @owner when
@@ -272,7 +268,7 @@ soup_buffer_copy (SoupBuffer *buffer)
 
 	/* For non-TEMPORARY buffers, this is just a ref */
 	if (priv->use != SOUP_MEMORY_TEMPORARY) {
-		g_atomic_int_inc (&priv->refcount);
+		priv->refcount++;
 		return buffer;
 	}
 
@@ -305,40 +301,28 @@ soup_buffer_free (SoupBuffer *buffer)
 {
 	SoupBufferPrivate *priv = (SoupBufferPrivate *)buffer;
 
-	if (!g_atomic_int_dec_and_test (&priv->refcount))
-		return;
-
-	if (priv->owner_dnotify)
-		priv->owner_dnotify (priv->owner);
-	g_slice_free (SoupBufferPrivate, priv);
+	if (!--priv->refcount) {
+		if (priv->owner_dnotify)
+			priv->owner_dnotify (priv->owner);
+		g_slice_free (SoupBufferPrivate, priv);
+	}
 }
 
-/**
- * soup_buffer_get_as_bytes:
- * @buffer: a #SoupBuffer
- *
- * Creates a #GBytes pointing to the same memory as @buffer. The
- * #GBytes will hold a reference on @buffer to ensure that it is not
- * freed while the #GBytes is still valid.
- *
- * Returns: (transfer full): a new #GBytes which has the same content
- * as the #SoupBuffer.
- *
- * Since: 2.40
- */
-GBytes *
-soup_buffer_get_as_bytes (SoupBuffer *buffer)
+GType
+soup_buffer_get_type (void)
 {
-	SoupBuffer *copy;
+	static volatile gsize type_volatile = 0;
 
-	copy = soup_buffer_copy (buffer);
-	return g_bytes_new_with_free_func (copy->data, copy->length,
-					   (GDestroyNotify)soup_buffer_free,
-					   copy);
+	if (g_once_init_enter (&type_volatile)) {
+		GType type = g_boxed_type_register_static (
+			g_intern_static_string ("SoupBuffer"),
+			(GBoxedCopyFunc) soup_buffer_copy,
+			(GBoxedFreeFunc) soup_buffer_free);
+		g_once_init_leave (&type_volatile, type);
+	}
+	return type_volatile;
 }
 
-G_DEFINE_BOXED_TYPE (SoupBuffer, soup_buffer, soup_buffer_copy, soup_buffer_free)
-
 
 /**
  * SoupMessageBody:
@@ -400,30 +384,27 @@ soup_message_body_new (void)
  * @body may be discarded when they are no longer needed.
  *
  * In particular, if you set this flag to %FALSE on an "incoming"
- * message body (that is, the #SoupMessage:response_body of a
- * client-side message, or #SoupMessage:request_body of a server-side
- * message), this will cause each chunk of the body to be discarded
- * after its corresponding #SoupMessage::got_chunk signal is emitted.
- * (This is equivalent to setting the deprecated
- * %SOUP_MESSAGE_OVERWRITE_CHUNKS flag on the message.)
+ * message body (that is, the %response_body of a client-side message,
+ * or %request_body of a server-side message), this will cause each
+ * chunk of the body to be discarded after its corresponding
+ * #SoupMessage::got_chunk signal is emitted. (This is equivalent to
+ * setting the deprecated %SOUP_MESSAGE_OVERWRITE_CHUNKS flag on the
+ * message.)
  *
- * If you set this flag to %FALSE on the #SoupMessage:response_body of
- * a server-side message, it will cause each chunk of the body to be
+ * If you set this flag to %FALSE on the %response_body of a
+ * server-side message, it will cause each chunk of the body to be
  * discarded after its corresponding #SoupMessage::wrote_chunk signal
  * is emitted.
  *
- * If you set the flag to %FALSE on the #SoupMessage:request_body of a
+ * (If you set the flag to %FALSE on the %request_body of a
  * client-side message, it will block the accumulation of chunks into
- * @body's %data field, but it will not normally cause the chunks to
- * be discarded after being written like in the server-side
- * #SoupMessage:response_body case, because the request body needs to
- * be kept around in case the request needs to be sent a second time
- * due to redirection or authentication. However, if you set the
- * %SOUP_MESSAGE_CAN_REBUILD flag on the message, then the chunks will
- * be discarded, and you will be responsible for recreating the
- * request body after the #SoupMessage::restarted signal is emitted.
+ * @body's %data field, but it will not cause the chunks to be
+ * discarded after being written like in the server-side
+ * %response_body case, because the request body needs to be kept
+ * around in case the request needs to be sent a second time due to
+ * redirection or authentication.)
  *
- * Since: 2.24
+ * Since: 2.4.1
  **/
 void
 soup_message_body_set_accumulate (SoupMessageBody *body,
@@ -443,7 +424,7 @@ soup_message_body_set_accumulate (SoupMessageBody *body,
  *
  * Return value: the accumulate flag for @body.
  *
- * Since: 2.24
+ * Since: 2.4.1
  **/
 gboolean
 soup_message_body_get_accumulate (SoupMessageBody *body)
@@ -492,18 +473,19 @@ soup_message_body_append (SoupMessageBody *body, SoupMemoryUse use,
 }
 
 /**
- * soup_message_body_append_take: (rename-to soup_message_body_append)
+ * soup_message_body_append_take:
  * @body: a #SoupMessageBody
  * @data: (array length=length) (transfer full): data to append
  * @length: length of @data
  *
  * Appends @length bytes from @data to @body.
  *
- * This function is exactly equivalent to soup_message_body_append()
+ * This function is exactly equivalent to soup_message_body_apppend()
  * with %SOUP_MEMORY_TAKE as second argument; it exists mainly for
  * convenience and simplifying language bindings.
  *
  * Since: 2.32
+ * Rename to: soup_message_body_append
  **/
 void
 soup_message_body_append_take (SoupMessageBody *body,
@@ -539,8 +521,11 @@ void
 soup_message_body_truncate (SoupMessageBody *body)
 {
 	SoupMessageBodyPrivate *priv = (SoupMessageBodyPrivate *)body;
+	GSList *iter;
 
-	g_slist_free_full (priv->chunks, (GDestroyNotify)soup_buffer_free);
+	for (iter = priv->chunks; iter; iter = iter->next)
+		soup_buffer_free (iter->data);
+	g_slist_free (priv->chunks);
 	priv->chunks = priv->last = NULL;
 	priv->base_offset = 0;
 
@@ -628,7 +613,7 @@ soup_message_body_flatten (SoupMessageBody *body)
  * @body may still potentially have more data, but that data is not
  * currently available).
  *
- * Return value: (nullable): a #SoupBuffer, or %NULL.
+ * Return value: a #SoupBuffer, or %NULL.
  **/
 SoupBuffer *
 soup_message_body_get_chunk (SoupMessageBody *body, goffset offset)
@@ -671,7 +656,7 @@ soup_message_body_get_chunk (SoupMessageBody *body, goffset offset)
  * This is a low-level method which you should not normally need to
  * use.
  *
- * Since: 2.24
+ * Since: 2.4.1
  **/
 void
 soup_message_body_got_chunk (SoupMessageBody *body, SoupBuffer *chunk)
@@ -698,7 +683,7 @@ soup_message_body_got_chunk (SoupMessageBody *body, SoupBuffer *chunk)
  * there are further restrictions on its proper use which are not
  * documented here.
  *
- * Since: 2.24
+ * Since: 2.4.1
  **/
 void
 soup_message_body_wrote_chunk (SoupMessageBody *body, SoupBuffer *chunk)
@@ -726,7 +711,7 @@ soup_message_body_copy (SoupMessageBody *body)
 {
 	SoupMessageBodyPrivate *priv = (SoupMessageBodyPrivate *)body;
 
-	g_atomic_int_inc (&priv->ref_count);
+	priv->ref_count++;
 	return body;
 }
 
@@ -742,11 +727,23 @@ soup_message_body_free (SoupMessageBody *body)
 {
 	SoupMessageBodyPrivate *priv = (SoupMessageBodyPrivate *)body;
 
-	if (!g_atomic_int_dec_and_test (&priv->ref_count))
-		return;
-
-	soup_message_body_truncate (body);
-	g_slice_free (SoupMessageBodyPrivate, priv);
+	if (--priv->ref_count == 0) {
+		soup_message_body_truncate (body);
+		g_slice_free (SoupMessageBodyPrivate, priv);
+	}
 }
 
-G_DEFINE_BOXED_TYPE (SoupMessageBody, soup_message_body, soup_message_body_copy, soup_message_body_free)
+GType
+soup_message_body_get_type (void)
+{
+	static volatile gsize type_volatile = 0;
+
+	if (g_once_init_enter (&type_volatile)) {
+		GType type = g_boxed_type_register_static (
+			g_intern_static_string ("SoupMessageBody"),
+			(GBoxedCopyFunc) soup_message_body_copy,
+			(GBoxedFreeFunc) soup_message_body_free);
+		g_once_init_leave (&type_volatile, type);
+	}
+	return type_volatile;
+}
