@@ -5,15 +5,12 @@
  * Copyright (C) 2007, 2008 Red Hat, Inc.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
+#include <stdio.h>
 #include <string.h>
 
 #include "soup-message-headers.h"
-#include "soup.h"
-#include "soup-misc-private.h"
+#include "soup-headers.h"
+#include "soup-misc.h"
 
 /**
  * SECTION:soup-message-headers
@@ -24,12 +21,6 @@
  * with a request or response.
  **/
 
-/**
- * SoupMessageHeaders:
- *
- * The HTTP message headers associated with a request or response.
- */
-
 /**
  * SoupMessageHeadersType:
  * @SOUP_MESSAGE_HEADERS_REQUEST: request headers
@@ -90,7 +81,7 @@ soup_message_headers_new (SoupMessageHeadersType type)
 static SoupMessageHeaders *
 soup_message_headers_copy (SoupMessageHeaders *hdrs)
 {
-	g_atomic_int_inc (&hdrs->ref_count);
+	hdrs->ref_count++;
 	return hdrs;
 }
 
@@ -103,31 +94,28 @@ soup_message_headers_copy (SoupMessageHeaders *hdrs)
 void
 soup_message_headers_free (SoupMessageHeaders *hdrs)
 {
-	if (!g_atomic_int_dec_and_test (&hdrs->ref_count))
-		return;
-
-	soup_message_headers_clear (hdrs);
-	g_array_free (hdrs->array, TRUE);
-	g_clear_pointer (&hdrs->concat, g_hash_table_destroy);
-	g_slice_free (SoupMessageHeaders, hdrs);
+	if (--hdrs->ref_count == 0) {
+		soup_message_headers_clear (hdrs);
+		g_array_free (hdrs->array, TRUE);
+		if (hdrs->concat)
+			g_hash_table_destroy (hdrs->concat);
+		g_slice_free (SoupMessageHeaders, hdrs);
+	}
 }
 
-G_DEFINE_BOXED_TYPE (SoupMessageHeaders, soup_message_headers, soup_message_headers_copy, soup_message_headers_free)
-
-/**
- * soup_message_headers_get_headers_type:
- * @hdrs: a #SoupMessageHeaders
- *
- * Gets the type of headers.
- *
- * Return value: the header's type.
- *
- * Since: 2.50
- **/
-SoupMessageHeadersType
-soup_message_headers_get_headers_type (SoupMessageHeaders *hdrs)
+GType
+soup_message_headers_get_type (void)
 {
-	return hdrs->type;
+	static volatile gsize type_volatile = 0;
+
+	if (g_once_init_enter (&type_volatile)) {
+		GType type = g_boxed_type_register_static (
+			g_intern_static_string ("SoupMessageHeaders"),
+			(GBoxedCopyFunc) soup_message_headers_copy,
+			(GBoxedFreeFunc) soup_message_headers_free);
+		g_once_init_leave (&type_volatile, type);
+	}
+	return type_volatile;
 }
 
 /**
@@ -140,7 +128,7 @@ void
 soup_message_headers_clear (SoupMessageHeaders *hdrs)
 {
 	SoupHeader *hdr_array = (SoupHeader *)hdrs->array->data;
-	guint i;
+	int i;
 
 	for (i = 0; i < hdrs->array->len; i++)
 		g_free (hdr_array[i].value);
@@ -308,7 +296,7 @@ soup_message_headers_remove (SoupMessageHeaders *hdrs, const char *name)
 /**
  * soup_message_headers_get_one:
  * @hdrs: a #SoupMessageHeaders
- * @name: (in): header name
+ * @name: header name
  * 
  * Gets the value of header @name in @hdrs. Use this for headers whose
  * values are <emphasis>not</emphasis> comma-delimited lists, and
@@ -320,9 +308,9 @@ soup_message_headers_remove (SoupMessageHeaders *hdrs, const char *name)
  * whichever one makes libsoup most compatible with other HTTP
  * implementations.)
  *
- * Return value: (nullable) (transfer none): the header's value or %NULL if not found.
+ * Return value: the header's value or %NULL if not found.
  *
- * Since: 2.28
+ * Since: 2.26.1
  **/
 const char *
 soup_message_headers_get_one (SoupMessageHeaders *hdrs, const char *name)
@@ -340,59 +328,6 @@ soup_message_headers_get_one (SoupMessageHeaders *hdrs, const char *name)
 	return (index == -1) ? NULL : hdr_array[index].value;
 }
 
-/**
- * soup_message_headers_header_contains:
- * @hdrs: a #SoupMessageHeaders
- * @name: header name
- * @token: token to look for
- *
- * Checks whether the list-valued header @name is present in @hdrs,
- * and contains a case-insensitive match for @token.
- *
- * (If @name is present in @hdrs, then this is equivalent to calling
- * soup_header_contains() on its value.)
- *
- * Return value: %TRUE if the header is present and contains @token,
- *   %FALSE otherwise.
- *
- * Since: 2.50
- **/
-gboolean
-soup_message_headers_header_contains (SoupMessageHeaders *hdrs, const char *name, const char *token)
-{
-	const char *value;
-
-	value = soup_message_headers_get_list (hdrs, name);
-	if (!value)
-		return FALSE;
-	return soup_header_contains (value, token);
-}
-
-/**
- * soup_message_headers_header_equals:
- * @hdrs: a #SoupMessageHeaders
- * @name: header name
- * @value: expected value
- *
- * Checks whether the header @name is present in @hdrs and is
- * (case-insensitively) equal to @value.
- *
- * Return value: %TRUE if the header is present and its value is
- *   @value, %FALSE otherwise.
- *
- * Since: 2.50
- **/
-gboolean
-soup_message_headers_header_equals (SoupMessageHeaders *hdrs, const char *name, const char *value)
-{
-        const char *internal_value;
-
-        internal_value = soup_message_headers_get_list (hdrs, name);
-	if (!internal_value)
-		return FALSE;
-        return !g_ascii_strcasecmp (internal_value, value);
-}
-
 /**
  * soup_message_headers_get_list:
  * @hdrs: a #SoupMessageHeaders
@@ -411,9 +346,9 @@ soup_message_headers_header_equals (SoupMessageHeaders *hdrs, const char *name,
  * transformation is allowed, and so an upstream proxy could do the
  * same thing.
  * 
- * Return value: (nullable) (transfer none): the header's value or %NULL if not found.
+ * Return value: the header's value or %NULL if not found.
  *
- * Since: 2.28
+ * Since: 2.26.1
  **/
 const char *
 soup_message_headers_get_list (SoupMessageHeaders *hdrs, const char *name)
@@ -467,7 +402,7 @@ soup_message_headers_get_list (SoupMessageHeaders *hdrs, const char *name)
  * soup_message_headers_get_list() were introduced, so callers can
  * explicitly state which behavior they are expecting.
  *
- * Return value: (nullable): as with soup_message_headers_get_list().
+ * Return value: as with soup_message_headers_get_list().
  * 
  * Deprecated: Use soup_message_headers_get_one() or
  * soup_message_headers_get_list() instead.
@@ -580,14 +515,14 @@ soup_message_headers_foreach (SoupMessageHeaders *hdrs,
 			      gpointer            user_data)
 {
 	SoupHeader *hdr_array = (SoupHeader *)hdrs->array->data;
-	guint i;
+	int i;
 
 	for (i = 0; i < hdrs->array->len; i++)
 		func (hdr_array[i].name, hdr_array[i].value, user_data);
 }
 
 
-G_LOCK_DEFINE_STATIC (header_pool);
+static GStaticMutex header_pool_mutex = G_STATIC_MUTEX_INIT;
 static GHashTable *header_pool, *header_setters;
 
 static void transfer_encoding_setter (SoupMessageHeaders *, const char *);
@@ -614,7 +549,7 @@ intern_header_name (const char *name, SoupHeaderSetter *setter)
 {
 	const char *interned;
 
-	G_LOCK (header_pool);
+	g_static_mutex_lock (&header_pool_mutex);
 
 	if (!header_pool) {
 		header_pool = g_hash_table_new (soup_str_case_hash, soup_str_case_equal);
@@ -637,7 +572,7 @@ intern_header_name (const char *name, SoupHeaderSetter *setter)
 	if (setter)
 		*setter = g_hash_table_lookup (header_setters, interned);
 
-	G_UNLOCK (header_pool);
+	g_static_mutex_unlock (&header_pool_mutex);
 	return interned;
 }
 
@@ -664,15 +599,9 @@ static void
 transfer_encoding_setter (SoupMessageHeaders *hdrs, const char *value)
 {
 	if (value) {
-		/* "identity" is a wrong value according to RFC errata 408,
-		 * and RFC 7230 does not list it as valid transfer-coding.
-		 * Nevertheless, the obsolete RFC 2616 stated "identity"
-		 * as valid, so we can't handle it as unrecognized here
-		 * for compatibility reasons.
-		 */
 		if (g_ascii_strcasecmp (value, "chunked") == 0)
 			hdrs->encoding = SOUP_ENCODING_CHUNKED;
-		else if (g_ascii_strcasecmp (value, "identity") != 0)
+		else
 			hdrs->encoding = SOUP_ENCODING_UNRECOGNIZED;
 	} else
 		hdrs->encoding = -1;
@@ -805,13 +734,8 @@ soup_message_headers_set_encoding (SoupMessageHeaders *hdrs,
 goffset
 soup_message_headers_get_content_length (SoupMessageHeaders *hdrs)
 {
-	SoupEncoding encoding;
-
-	encoding = soup_message_headers_get_encoding (hdrs);
-	if (encoding == SOUP_ENCODING_CONTENT_LENGTH)
-		return hdrs->content_length;
-	else
-		return 0;
+	return (hdrs->encoding == SOUP_ENCODING_CONTENT_LENGTH) ?
+		hdrs->content_length : 0;
 }
 
 /**
@@ -837,8 +761,8 @@ soup_message_headers_set_content_length (SoupMessageHeaders *hdrs,
 {
 	char length[128];
 
-	g_snprintf (length, sizeof (length), "%" G_GUINT64_FORMAT,
-		    content_length);
+	snprintf (length, sizeof (length), "%" G_GUINT64_FORMAT,
+		  content_length);
 	soup_message_headers_remove (hdrs, "Transfer-Encoding");
 	soup_message_headers_replace (hdrs, "Content-Length", length);
 }
@@ -915,17 +839,17 @@ soup_message_headers_set_expectations (SoupMessageHeaders *hdrs,
  * Represents a byte range as used in the Range header.
  *
  * If @end is non-negative, then @start and @end represent the bounds
- * of of the range, counting from 0. (Eg, the first 500 bytes would be
- * represented as @start = 0 and @end = 499.)
+ * of of the range, counting from %0. (Eg, the first 500 bytes would be
+ * represented as @start = %0 and @end = %499.)
  *
- * If @end is -1 and @start is non-negative, then this represents a
+ * If @end is %-1 and @start is non-negative, then this represents a
  * range starting at @start and ending with the last byte of the
  * requested resource body. (Eg, all but the first 500 bytes would be
- * @start = 500, and @end = -1.)
+ * @start = %500, and @end = %-1.)
  *
- * If @end is -1 and @start is negative, then it represents a "suffix
+ * If @end is %-1 and @start is negative, then it represents a "suffix
  * range", referring to the last -@start bytes of the resource body.
- * (Eg, the last 500 bytes would be @start = -500 and @end = -1.)
+ * (Eg, the last 500 bytes would be @start = %-500 and @end = %-1.)
  *
  * Since: 2.26
  **/
@@ -939,39 +863,56 @@ sort_ranges (gconstpointer a, gconstpointer b)
 	return ra->start - rb->start;
 }
 
-/* like soup_message_headers_get_ranges(), except it returns:
- *   SOUP_STATUS_OK if there is no Range or it should be ignored.
- *   SOUP_STATUS_PARTIAL_CONTENT if there is at least one satisfiable range.
- *   SOUP_STATUS_REQUESTED_RANGE_NOT_SATISFIABLE if @check_satisfiable
- *     is %TRUE and the request is not satisfiable given @total_length.
- */
-guint
-soup_message_headers_get_ranges_internal (SoupMessageHeaders  *hdrs,
-					  goffset              total_length,
-					  gboolean             check_satisfiable,
-					  SoupRange          **ranges,
-					  int                 *length)
+/**
+ * soup_message_headers_get_ranges:
+ * @hdrs: a #SoupMessageHeaders
+ * @total_length: the total_length of the response body
+ * @ranges: (out): return location for an array of #SoupRange
+ * @length: the length of the returned array
+ *
+ * Parses @hdrs's Range header and returns an array of the requested
+ * byte ranges. The returned array must be freed with
+ * soup_message_headers_free_ranges().
+ *
+ * If @total_length is non-0, its value will be used to adjust the
+ * returned ranges to have explicit start and end values, and the
+ * returned ranges will be sorted and non-overlapping. If
+ * @total_length is 0, then some ranges may have an end value of -1,
+ * as described under #SoupRange, and some of the ranges may be
+ * redundant.
+ *
+ * Return value: %TRUE if @hdrs contained a "Range" header containing
+ * byte ranges which could be parsed, %FALSE otherwise (in which case
+ * @range and @length will not be set).
+ *
+ * Since: 2.26
+ **/
+gboolean
+soup_message_headers_get_ranges (SoupMessageHeaders  *hdrs,
+				 goffset              total_length,
+				 SoupRange          **ranges,
+				 int                 *length)
 {
 	const char *range = soup_message_headers_get_one (hdrs, "Range");
 	GSList *range_list, *r;
 	GArray *array;
 	char *spec, *end;
-	guint status = SOUP_STATUS_OK;
+	int i;
 
 	if (!range || strncmp (range, "bytes", 5) != 0)
-		return status;
+		return FALSE;
 
 	range += 5;
 	while (g_ascii_isspace (*range))
 		range++;
 	if (*range++ != '=')
-		return status;
+		return FALSE;
 	while (g_ascii_isspace (*range))
 		range++;
 
 	range_list = soup_header_parse_list (range);
 	if (!range_list)
-		return status;
+		return FALSE;
 
 	array = g_array_new (FALSE, FALSE, sizeof (SoupRange));
 	for (r = range_list; r; r = r->next) {
@@ -985,37 +926,23 @@ soup_message_headers_get_ranges_internal (SoupMessageHeaders  *hdrs,
 			cur.start = g_ascii_strtoull (spec, &end, 10);
 			if (*end == '-')
 				end++;
-			if (*end) {
+			if (*end)
 				cur.end = g_ascii_strtoull (end, &end, 10);
-				if (cur.end < cur.start) {
-					status = SOUP_STATUS_OK;
-					break;
-				}
-			} else
+			else
 				cur.end = total_length - 1;
 		}
 		if (*end) {
-			status = SOUP_STATUS_OK;
-			break;
-		} else if (check_satisfiable && cur.start >= total_length) {
-			if (status == SOUP_STATUS_OK)
-				status = SOUP_STATUS_REQUESTED_RANGE_NOT_SATISFIABLE;
-			continue;
+			g_array_free (array, TRUE);
+			soup_header_free_list (range_list);
+			return FALSE;
 		}
 
 		g_array_append_val (array, cur);
-		status = SOUP_STATUS_PARTIAL_CONTENT;
 	}
+
 	soup_header_free_list (range_list);
 
-	if (status != SOUP_STATUS_PARTIAL_CONTENT) {
-		g_array_free (array, TRUE);
-		return status;
-	}
-
 	if (total_length) {
-		guint i;
-
 		g_array_sort (array, sort_ranges);
 		for (i = 1; i < array->len; i++) {
 			SoupRange *cur = &((SoupRange *)array->data)[i];
@@ -1032,63 +959,7 @@ soup_message_headers_get_ranges_internal (SoupMessageHeaders  *hdrs,
 	*length = array->len;
 
 	g_array_free (array, FALSE);
-	return SOUP_STATUS_PARTIAL_CONTENT;
-}
-
-/**
- * soup_message_headers_get_ranges:
- * @hdrs: a #SoupMessageHeaders
- * @total_length: the total_length of the response body
- * @ranges: (out) (array length=length): return location for an array
- * of #SoupRange
- * @length: the length of the returned array
- *
- * Parses @hdrs's Range header and returns an array of the requested
- * byte ranges. The returned array must be freed with
- * soup_message_headers_free_ranges().
- *
- * If @total_length is non-0, its value will be used to adjust the
- * returned ranges to have explicit start and end values, and the
- * returned ranges will be sorted and non-overlapping. If
- * @total_length is 0, then some ranges may have an end value of -1,
- * as described under #SoupRange, and some of the ranges may be
- * redundant.
- *
- * Beware that even if given a @total_length, this function does not
- * check that the ranges are satisfiable.
- *
- * <note><para>
- * #SoupServer has built-in handling for range requests. If your
- * server handler returns a %SOUP_STATUS_OK response containing the
- * complete response body (rather than pausing the message and
- * returning some of the response body later), and there is a Range
- * header in the request, then libsoup will automatically convert the
- * response to a %SOUP_STATUS_PARTIAL_CONTENT response containing only
- * the range(s) requested by the client.
- *
- * The only time you need to process the Range header yourself is if
- * either you need to stream the response body rather than returning
- * it all at once, or you do not already have the complete response
- * body available, and only want to generate the parts that were
- * actually requested by the client.
- * </para></note>
- *
- * Return value: %TRUE if @hdrs contained a syntactically-valid
- * "Range" header, %FALSE otherwise (in which case @range and @length
- * will not be set).
- *
- * Since: 2.26
- **/
-gboolean
-soup_message_headers_get_ranges (SoupMessageHeaders  *hdrs,
-				 goffset              total_length,
-				 SoupRange          **ranges,
-				 int                 *length)
-{
-	guint status;
-
-	status = soup_message_headers_get_ranges_internal (hdrs, total_length, FALSE, ranges, length);
-	return status == SOUP_STATUS_PARTIAL_CONTENT;
+	return TRUE;
 }
 
 /**
@@ -1176,10 +1047,10 @@ soup_message_headers_set_range (SoupMessageHeaders  *hdrs,
 /**
  * soup_message_headers_get_content_range:
  * @hdrs: a #SoupMessageHeaders
- * @start: (out): return value for the start of the range
- * @end: (out): return value for the end of the range
- * @total_length: (out) (optional): return value for the total length of the
- * resource, or %NULL if you don't care.
+ * @start: return value for the start of the range
+ * @end: return value for the end of the range
+ * @total_length: return value for the total length of the resource,
+ * or %NULL if you don't care.
  *
  * Parses @hdrs's Content-Range header and returns it in @start,
  * @end, and @total_length. If the total length field in the header
@@ -1238,12 +1109,6 @@ soup_message_headers_get_content_range (SoupMessageHeaders  *hdrs,
  * (Note that @total_length is the total length of the entire resource
  * that this is a range of, not simply @end - @start + 1.)
  *
- * <note><para>
- * #SoupServer has built-in handling for range requests, and you do
- * not normally need to call this function youself. See
- * soup_message_headers_get_ranges() for more details.
- * </para></note>
- *
  * Since: 2.26
  **/
 void
@@ -1272,37 +1137,26 @@ parse_content_foo (SoupMessageHeaders *hdrs, const char *header_name,
 {
 	const char *header;
 	char *semi;
-        char *equal;
 
 	header = soup_message_headers_get_one (hdrs, header_name);
 	if (!header)
 		return FALSE;
 
-        /* Some websites send an invalid disposition that only contains parameters;
-         * We can be flexible about handling these by detecting if the first word
-         * is a parameter (foo=bar). */
-        equal = strchr (header, '=');
-        semi = strchr (header, ';');
-        if (g_ascii_strcasecmp (header_name, "Content-Disposition") == 0 &&
-            (equal && (!semi || (equal < semi)))) {
-                semi = (char *)header;
-                if (foo)
-                        *foo = NULL;
-        } else if (foo) {
-                *foo = g_strdup (header);
-                semi = strchr (*foo, ';');
-                if (semi) {
-                        char *p = semi;
+	if (foo) {
+		*foo = g_strdup (header);
+		semi = strchr (*foo, ';');
+		if (semi) {
+			char *p = semi;
 
-                        *semi++ = '\0';
-                        while (p - 1 > *foo && g_ascii_isspace(p[-1]))
-                                *(--p) = '\0';
-                }
-        } else {
-                /* Skip type, we don't store it */
-                if (semi)
-                        semi++;
-        }
+			*semi++ = '\0';
+			while (p - 1 > *foo && g_ascii_isspace(p[-1]))
+				*(--p) = '\0';
+		}
+	} else {
+		semi = strchr (header, ';');
+		if (semi)
+			semi++;
+	}
 
 	if (!params)
 		return TRUE;
@@ -1341,37 +1195,33 @@ static void
 content_type_setter (SoupMessageHeaders *hdrs, const char *value)
 {
 	g_free (hdrs->content_type);
-	hdrs->content_type = NULL;
-
 	if (value) {
-		char *content_type = NULL, *p;
+		char *content_type, *p;
 
 		parse_content_foo (hdrs, "Content-Type", &content_type, NULL);
-		g_return_if_fail (content_type != NULL);
-
 		p = strpbrk (content_type, " /");
-		if (!p || *p != '/' || strpbrk (p + 1, " /"))
+		if (!p || *p != '/' || strpbrk (p + 1, " /")) {
 			g_free (content_type);
-		else
+			hdrs->content_type = NULL;
+		} else
 			hdrs->content_type = content_type;
-	}
+	} else
+		hdrs->content_type = NULL;
 }
 
 /**
  * soup_message_headers_get_content_type:
  * @hdrs: a #SoupMessageHeaders
- * @params: (out) (element-type utf8 utf8) (allow-none) (transfer full):
- *   return location for the Content-Type parameters (eg, "charset"), or
- *   %NULL
+ * @params: (out) (allow-none) (transfer full): return location for
+ * the Content-Type parameters (eg, "charset"), or %NULL
  *
  * Looks up the "Content-Type" header in @hdrs, parses it, and returns
  * its value in *@content_type and *@params. @params can be %NULL if you
  * are only interested in the content type itself.
  *
- * Return value: (nullable): a string with the value of the
- * "Content-Type" header or %NULL if @hdrs does not contain that
- * header or it cannot be parsed (in which case *@params will be
- * unchanged).
+ * Return value: a string with the value of the "Content-Type" header
+ * or NULL if @hdrs does not contain that header or it cannot be
+ * parsed (in which case *@params will be unchanged).
  *
  * Since: 2.26
  **/
@@ -1457,7 +1307,7 @@ soup_message_headers_get_content_disposition (SoupMessageHeaders  *hdrs,
 		char *filename = strrchr (orig_value, '/');
 
 		if (filename)
-			g_hash_table_insert (*params, g_strdup (orig_key), filename + 1);
+			g_hash_table_insert (*params, orig_key, filename + 1);
 	}
 	return TRUE;
 }
