@@ -5,16 +5,14 @@
  * Copyright 1999-2003 Ximian, Inc.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
+#include <ctype.h>
 #include <string.h>
 #include <stdlib.h>
 
 #include "soup-uri.h"
-#include "soup.h"
 #include "soup-misc-private.h"
+#include "soup-form.h"
+#include "soup-misc.h"
 
 /**
  * SECTION:soup-uri
@@ -33,7 +31,7 @@
  * @scheme: the URI scheme (eg, "http")
  * @user: a username, or %NULL
  * @password: a password, or %NULL
- * @host: the hostname or IP address, or %NULL
+ * @host: the hostname or IP address
  * @port: the port number on @host
  * @path: the path on @host
  * @query: a query for @path, or %NULL
@@ -82,21 +80,6 @@
  * as well, but this would be more annoying than useful.)
  **/
 
-/**
- * SOUP_URI_IS_VALID:
- * @uri: a #SoupURI
- *
- * Tests whether @uri is a valid #SoupURI; that is, that it is non-%NULL
- * and its @scheme and @path members are also non-%NULL.
- *
- * This macro does not check whether http and https URIs have a non-%NULL
- * @host member.
- *
- * Return value: %TRUE if @uri is valid for use.
- *
- * Since: 2.38
- **/
-
 /**
  * SOUP_URI_VALID_FOR_HTTP:
  * @uri: a #SoupURI
@@ -109,82 +92,12 @@
  * Since: 2.24
  **/
 
-/**
- * SOUP_URI_SCHEME_HTTP:
- *
- * "http" as an interned string; you can compare this directly to a
- * #SoupURI's <literal>scheme</literal> field using
- * <literal>==</literal>.
- */
-/**
- * SOUP_URI_SCHEME_HTTPS:
- *
- * "https" as an interned string; you can compare this directly to a
- * #SoupURI's <literal>scheme</literal> field using
- * <literal>==</literal>.
- */
-/**
- * SOUP_URI_SCHEME_FTP:
- *
- * "ftp" as an interned string; you can compare this directly to a
- * #SoupURI's <literal>scheme</literal> field using
- * <literal>==</literal>.
- *
- * Since: 2.30
- */
-/**
- * SOUP_URI_SCHEME_FILE:
- *
- * "file" as an interned string; you can compare this directly to a
- * #SoupURI's <literal>scheme</literal> field using
- * <literal>==</literal>.
- *
- * Since: 2.30
- */
-/**
- * SOUP_URI_SCHEME_DATA:
- *
- * "data" as an interned string; you can compare this directly to a
- * #SoupURI's <literal>scheme</literal> field using
- * <literal>==</literal>.
- *
- * Since: 2.30
- */
-/**
- * SOUP_URI_SCHEME_RESOURCE:
- *
- * "data" as an interned string; you can compare this directly to a
- * #SoupURI's <literal>scheme</literal> field using
- * <literal>==</literal>.
- *
- * Since: 2.42
- */
-/**
- * SOUP_URI_SCHEME_WS:
- *
- * "ws" (WebSocket) as an interned string; you can compare this
- * directly to a #SoupURI's <literal>scheme</literal> field using
- * <literal>==</literal>.
- *
- * Since: 2.50
- */
-/**
- * SOUP_URI_SCHEME_WSS:
- *
- * "wss" (WebSocket over TLS) as an interned string; you can compare
- * this directly to a #SoupURI's <literal>scheme</literal> field using
- * <literal>==</literal>.
- *
- * Since: 2.50
- */
-
 static void append_uri_encoded (GString *str, const char *in, const char *extra_enc_chars);
 static char *uri_normalized_copy (const char *str, int length, const char *unescape_extra);
 
 gpointer _SOUP_URI_SCHEME_HTTP, _SOUP_URI_SCHEME_HTTPS;
-gpointer _SOUP_URI_SCHEME_WS, _SOUP_URI_SCHEME_WSS;
 gpointer _SOUP_URI_SCHEME_FTP;
-gpointer _SOUP_URI_SCHEME_FILE, _SOUP_URI_SCHEME_DATA, _SOUP_URI_SCHEME_RESOURCE;
+gpointer _SOUP_URI_SCHEME_FILE, _SOUP_URI_SCHEME_DATA;
 
 static inline const char *
 soup_uri_parse_scheme (const char *scheme, int len)
@@ -193,12 +106,6 @@ soup_uri_parse_scheme (const char *scheme, int len)
 		return SOUP_URI_SCHEME_HTTP;
 	} else if (len == 5 && !g_ascii_strncasecmp (scheme, "https", len)) {
 		return SOUP_URI_SCHEME_HTTPS;
-	} else if (len == 8 && !g_ascii_strncasecmp (scheme, "resource", len)) {
-		return SOUP_URI_SCHEME_RESOURCE;
-	} else if (len == 2 && !g_ascii_strncasecmp (scheme, "ws", len)) {
-		return SOUP_URI_SCHEME_WS;
-	} else if (len == 3 && !g_ascii_strncasecmp (scheme, "wss", len)) {
-		return SOUP_URI_SCHEME_WSS;
 	} else {
 		char *lower_scheme;
 
@@ -213,9 +120,9 @@ soup_uri_parse_scheme (const char *scheme, int len)
 static inline guint
 soup_scheme_default_port (const char *scheme)
 {
-	if (scheme == SOUP_URI_SCHEME_HTTP || scheme == SOUP_URI_SCHEME_WS)
+	if (scheme == SOUP_URI_SCHEME_HTTP)
 		return 80;
-	else if (scheme == SOUP_URI_SCHEME_HTTPS || scheme == SOUP_URI_SCHEME_WSS)
+	else if (scheme == SOUP_URI_SCHEME_HTTPS)
 		return 443;
 	else if (scheme == SOUP_URI_SCHEME_FTP)
 		return 21;
@@ -224,36 +131,23 @@ soup_scheme_default_port (const char *scheme)
 }
 
 /**
- * soup_uri_new_with_base: (constructor)
+ * soup_uri_new_with_base:
  * @base: a base URI
  * @uri_string: the URI
  *
  * Parses @uri_string relative to @base.
  *
- * Returns: a parsed #SoupURI.
+ * Return value: a parsed #SoupURI.
  **/
 SoupURI *
 soup_uri_new_with_base (SoupURI *base, const char *uri_string)
 {
-	SoupURI *uri, fixed_base;
+	SoupURI *uri;
 	const char *end, *hash, *colon, *at, *path, *question;
 	const char *p, *hostend;
 	gboolean remove_dot_segments = TRUE;
 	int len;
 
-	g_return_val_if_fail (uri_string != NULL, NULL);
-
-	/* Allow a %NULL path in @base, for compatibility */
-	if (base && base->scheme && !base->path) {
-		g_warn_if_fail (SOUP_URI_IS_VALID (base));
-
-		memcpy (&fixed_base, base, sizeof (SoupURI));
-		fixed_base.path = "";
-		base = &fixed_base;
-	}
-
-	g_return_val_if_fail (base == NULL || SOUP_URI_IS_VALID (base), NULL);
-
 	/* First some cleanup steps (which are supposed to all be no-ops,
 	 * but...). Skip initial whitespace, strip out internal tabs and
 	 * line breaks, and ignore trailing whitespace.
@@ -290,13 +184,10 @@ soup_uri_new_with_base (SoupURI *base, const char *uri_string)
 		end = hash;
 	}
 
-	/* Find scheme */
+	/* Find scheme: initial [a-z+.-]* substring until ":" */
 	p = uri_string;
-	while (p < end && (g_ascii_isalpha (*p) ||
-			   (p > uri_string && (g_ascii_isdigit (*p) ||
-					       *p == '.' ||
-					       *p == '+' ||
-					       *p == '-'))))
+	while (p < end && (g_ascii_isalnum (*p) ||
+			   *p == '.' || *p == '+' || *p == '-'))
 		p++;
 
 	if (p > uri_string && *p == ':') {
@@ -304,10 +195,8 @@ soup_uri_new_with_base (SoupURI *base, const char *uri_string)
 		uri_string = p + 1;
 	}
 
-	if (uri_string == end && !base && !uri->fragment) {
-		uri->path = g_strdup ("");
+	if (uri_string == end && !base && !uri->fragment)
 		return uri;
-        }
 
 	/* Check for authority */
 	if (strncmp (uri_string, "//", 2) == 0) {
@@ -320,23 +209,21 @@ soup_uri_new_with_base (SoupURI *base, const char *uri_string)
 		if (at && at < path) {
 			colon = strchr (uri_string, ':');
 			if (colon && colon < at) {
-				uri->password = soup_uri_decoded_copy (colon + 1,
-								       at - colon - 1, NULL);
+				uri->password = uri_decoded_copy (colon + 1,
+								  at - colon - 1);
 			} else {
 				uri->password = NULL;
 				colon = at;
 			}
 
-			uri->user = soup_uri_decoded_copy (uri_string,
-							   colon - uri_string, NULL);
+			uri->user = uri_decoded_copy (uri_string,
+						      colon - uri_string);
 			uri_string = at + 1;
 		} else
 			uri->user = uri->password = NULL;
 
 		/* Find host and port. */
 		if (*uri_string == '[') {
-			const char *pct;
-
 			uri_string++;
 			hostend = strchr (uri_string, ']');
 			if (!hostend || hostend > path) {
@@ -347,20 +234,13 @@ soup_uri_new_with_base (SoupURI *base, const char *uri_string)
 				colon = hostend + 1;
 			else
 				colon = NULL;
-
-			pct = memchr (uri_string, '%', hostend - uri_string);
-			if (!pct || (pct[1] == '2' && pct[2] == '5')) {
-				uri->host = soup_uri_decoded_copy (uri_string,
-								   hostend - uri_string, NULL);
-			} else
-				uri->host = g_strndup (uri_string, hostend - uri_string);
 		} else {
 			colon = memchr (uri_string, ':', path - uri_string);
 			hostend = colon ? colon : path;
-			uri->host = soup_uri_decoded_copy (uri_string,
-							   hostend - uri_string, NULL);
 		}
 
+		uri->host = uri_decoded_copy (uri_string, hostend - uri_string);
+
 		if (colon && colon != path - 1) {
 			char *portend;
 			uri->port = strtoul (colon + 1, &portend, 10);
@@ -453,7 +333,7 @@ soup_uri_new_with_base (SoupURI *base, const char *uri_string)
 		}
 		/* Remove "<segment>/.." at end where <segment> != ".." */
 		q = strrchr (uri->path, '/');
-		if (q && q != uri->path && !strcmp (q, "/..")) {
+		if (q && !strcmp (q, "/..")) {
 			p = q - 1;
 			while (p > uri->path && *p != '/')
 				p--;
@@ -496,7 +376,7 @@ soup_uri_new_with_base (SoupURI *base, const char *uri_string)
 
 /**
  * soup_uri_new:
- * @uri_string: (allow-none): a URI
+ * @uri_string: a URI
  *
  * Parses an absolute URI.
  *
@@ -505,8 +385,7 @@ soup_uri_new_with_base (SoupURI *base, const char *uri_string)
  * call at least soup_uri_set_scheme() and soup_uri_set_path(), since
  * those fields are required.)
  *
- * Return value: (nullable): a #SoupURI, or %NULL if the given string
- *  was found to be invalid.
+ * Return value: a #SoupURI, or %NULL.
  **/
 SoupURI *
 soup_uri_new (const char *uri_string)
@@ -519,7 +398,7 @@ soup_uri_new (const char *uri_string)
 	uri = soup_uri_new_with_base (NULL, uri_string);
 	if (!uri)
 		return NULL;
-	if (!SOUP_URI_IS_VALID (uri)) {
+	if (!uri->scheme) {
 		soup_uri_free (uri);
 		return NULL;
 	}
@@ -528,17 +407,32 @@ soup_uri_new (const char *uri_string)
 }
 
 
+/**
+ * soup_uri_to_string:
+ * @uri: a #SoupURI
+ * @just_path_and_query: if %TRUE, output just the path and query portions
+ *
+ * Returns a string representing @uri.
+ *
+ * If @just_path_and_query is %TRUE, this concatenates the path and query
+ * together. That is, it constructs the string that would be needed in
+ * the Request-Line of an HTTP request for @uri.
+ *
+ * Return value: a string representing @uri, which the caller must free.
+ **/
 char *
-soup_uri_to_string_internal (SoupURI *uri, gboolean just_path_and_query,
-			     gboolean include_password, gboolean force_port)
+soup_uri_to_string (SoupURI *uri, gboolean just_path_and_query)
 {
 	GString *str;
 	char *return_result;
 
 	g_return_val_if_fail (uri != NULL, NULL);
-	g_warn_if_fail (SOUP_URI_IS_VALID (uri));
 
-	str = g_string_sized_new (40);
+	/* IF YOU CHANGE ANYTHING IN THIS FUNCTION, RUN
+	 * tests/uri-parsing AFTERWARD.
+	 */
+
+	str = g_string_sized_new (20);
 
 	if (uri->scheme && !just_path_and_query)
 		g_string_append_printf (str, "%s:", uri->scheme);
@@ -546,40 +440,22 @@ soup_uri_to_string_internal (SoupURI *uri, gboolean just_path_and_query,
 		g_string_append (str, "//");
 		if (uri->user) {
 			append_uri_encoded (str, uri->user, ":;@?/");
-			if (uri->password && include_password) {
-				g_string_append_c (str, ':');
-				append_uri_encoded (str, uri->password, ";@?/");
-			}
 			g_string_append_c (str, '@');
 		}
 		if (strchr (uri->host, ':')) {
-			const char *pct;
-
 			g_string_append_c (str, '[');
-			pct = strchr (uri->host, '%');
-			if (pct) {
-				g_string_append_printf (str, "%.*s%%25%s",
-							(int) (pct - uri->host),
-							uri->host, pct + 1);
-			} else
-				g_string_append (str, uri->host);
+			g_string_append (str, uri->host);
 			g_string_append_c (str, ']');
 		} else
 			append_uri_encoded (str, uri->host, ":/");
-		if (uri->port && (force_port || uri->port != soup_scheme_default_port (uri->scheme)))
+		if (uri->port && uri->port != soup_scheme_default_port (uri->scheme))
 			g_string_append_printf (str, ":%u", uri->port);
 		if (!uri->path && (uri->query || uri->fragment))
 			g_string_append_c (str, '/');
-		else if ((!uri->path || !*uri->path) &&
-			 (uri->scheme == SOUP_URI_SCHEME_HTTP ||
-			  uri->scheme == SOUP_URI_SCHEME_HTTPS))
-			g_string_append_c (str, '/');
 	}
 
 	if (uri->path && *uri->path)
 		g_string_append (str, uri->path);
-	else if (just_path_and_query)
-		g_string_append_c (str, '/');
 
 	if (uri->query) {
 		g_string_append_c (str, '?');
@@ -596,28 +472,6 @@ soup_uri_to_string_internal (SoupURI *uri, gboolean just_path_and_query,
 	return return_result;
 }
 
-/**
- * soup_uri_to_string:
- * @uri: a #SoupURI
- * @just_path_and_query: if %TRUE, output just the path and query portions
- *
- * Returns a string representing @uri.
- *
- * If @just_path_and_query is %TRUE, this concatenates the path and query
- * together. That is, it constructs the string that would be needed in
- * the Request-Line of an HTTP request for @uri.
- *
- * Note that the output will never contain a password, even if @uri
- * does.
- *
- * Return value: a string representing @uri, which the caller must free.
- **/
-char *
-soup_uri_to_string (SoupURI *uri, gboolean just_path_and_query)
-{
-	return soup_uri_to_string_internal (uri, just_path_and_query, FALSE, FALSE);
-}
-
 /**
  * soup_uri_copy:
  * @uri: a #SoupURI
@@ -632,7 +486,6 @@ soup_uri_copy (SoupURI *uri)
 	SoupURI *dup;
 
 	g_return_val_if_fail (uri != NULL, NULL);
-	g_warn_if_fail (SOUP_URI_IS_VALID (uri));
 
 	dup = g_slice_new0 (SoupURI);
 	dup->scheme   = uri->scheme;
@@ -669,11 +522,6 @@ parts_equal (const char *one, const char *two, gboolean insensitive)
 gboolean 
 soup_uri_equal (SoupURI *uri1, SoupURI *uri2)
 {
-	g_return_val_if_fail (uri1 != NULL, FALSE);
-	g_return_val_if_fail (uri2 != NULL, FALSE);
-	g_warn_if_fail (SOUP_URI_IS_VALID (uri1));
-	g_warn_if_fail (SOUP_URI_IS_VALID (uri2));
-
 	if (uri1->scheme != uri2->scheme                         ||
 	    uri1->port   != uri2->port                           ||
 	    !parts_equal (uri1->user, uri2->user, FALSE)         ||
@@ -741,8 +589,6 @@ soup_uri_encode (const char *part, const char *escape_extra)
 	GString *str;
 	char *encoded;
 
-	g_return_val_if_fail (part != NULL, NULL);
-
 	str = g_string_new (NULL);
 	append_uri_encoded (str, part, escape_extra);
 	encoded = str->str;
@@ -754,23 +600,16 @@ soup_uri_encode (const char *part, const char *escape_extra)
 #define XDIGIT(c) ((c) <= '9' ? (c) - '0' : ((c) & 0x4F) - 'A' + 10)
 #define HEXCHAR(s) ((XDIGIT (s[1]) << 4) + XDIGIT (s[2]))
 
-/* length must be set (e.g. from strchr()) such that [part, part + length]
- * contains no nul bytes */
 char *
-soup_uri_decoded_copy (const char *part, int length, int *decoded_length)
+uri_decoded_copy (const char *part, int length)
 {
 	unsigned char *s, *d;
-	char *decoded;
+	char *decoded = g_strndup (part, length);
 
-	g_return_val_if_fail (part != NULL, NULL);
-
-	decoded = g_strndup (part, length);
 	s = d = (unsigned char *)decoded;
 	do {
 		if (*s == '%') {
-			if (s[1] == '\0' ||
-			    s[2] == '\0' ||
-			    !g_ascii_isxdigit (s[1]) ||
+			if (!g_ascii_isxdigit (s[1]) ||
 			    !g_ascii_isxdigit (s[2])) {
 				*d++ = *s;
 				continue;
@@ -781,9 +620,6 @@ soup_uri_decoded_copy (const char *part, int length, int *decoded_length)
 			*d++ = *s;
 	} while (*s++);
 
-	if (decoded_length)
-		*decoded_length = d - (unsigned char *)decoded - 1;
-
 	return decoded;
 }
 
@@ -802,13 +638,9 @@ soup_uri_decoded_copy (const char *part, int length, int *decoded_length)
 char *
 soup_uri_decode (const char *part)
 {
-	g_return_val_if_fail (part != NULL, NULL);
-
-	return soup_uri_decoded_copy (part, strlen (part), NULL);
+	return uri_decoded_copy (part, strlen (part));
 }
 
-/* length must be set (e.g. from strchr()) such that [part, part + length]
- * contains no nul bytes */
 static char *
 uri_normalized_copy (const char *part, int length,
 		     const char *unescape_extra)
@@ -817,25 +649,20 @@ uri_normalized_copy (const char *part, int length,
 	char *normalized = g_strndup (part, length);
 	gboolean need_fixup = FALSE;
 
-	if (!unescape_extra)
-		unescape_extra = "";
-
 	s = d = (unsigned char *)normalized;
-	while (*s) {
+	do {
 		if (*s == '%') {
-			if (s[1] == '\0' ||
-			    s[2] == '\0' ||
-			    !g_ascii_isxdigit (s[1]) ||
+			if (!g_ascii_isxdigit (s[1]) ||
 			    !g_ascii_isxdigit (s[2])) {
-				*d++ = *s++;
+				*d++ = *s;
 				continue;
 			}
 
 			c = HEXCHAR (s);
 			if (soup_char_is_uri_unreserved (c) ||
-			    (c && strchr (unescape_extra, c))) {
+			    (unescape_extra && strchr (unescape_extra, c))) {
 				*d++ = c;
-				s += 3;
+				s += 2;
 			} else {
 				/* We leave it unchanged. We used to uppercase percent-encoded
 				 * triplets but we do not do it any more as RFC3986 Section 6.2.2.1
@@ -843,30 +670,27 @@ uri_normalized_copy (const char *part, int length,
 				 */
 				*d++ = *s++;
 				*d++ = *s++;
-				*d++ = *s++;
+				*d++ = *s;
 			}
 		} else {
-			if (!g_ascii_isgraph (*s) &&
-			    !strchr (unescape_extra, *s))
+			if (*s == ' ')
 				need_fixup = TRUE;
-			*d++ = *s++;
+			*d++ = *s;
 		}
-	}
-	*d = '\0';
+	} while (*s++);
 
 	if (need_fixup) {
 		GString *fixed;
+		char *sp, *p;
 
 		fixed = g_string_new (NULL);
-		s = (guchar *)normalized;
-		while (*s) {
-			if (g_ascii_isgraph (*s) ||
-			    strchr (unescape_extra, *s))
-				g_string_append_c (fixed, *s);
-			else
-				g_string_append_printf (fixed, "%%%02X", (int)*s);
-			s++;
+		p = normalized;
+		while ((sp = strchr (p, ' '))) {
+			g_string_append_len (fixed, p, sp - p);
+			g_string_append (fixed, "%20");
+			p = sp + 1;
 		}
+		g_string_append (fixed, p);
 		g_free (normalized);
 		normalized = g_string_free (fixed, FALSE);
 	}
@@ -877,11 +701,10 @@ uri_normalized_copy (const char *part, int length,
 /**
  * soup_uri_normalize:
  * @part: a URI part
- * @unescape_extra: (allow-none): reserved characters to unescape (or %NULL)
+ * @unescape_extra: reserved characters to unescape (or %NULL)
  *
  * %<!-- -->-decodes any "unreserved" characters (or characters in
- * @unescape_extra) in @part, and %<!-- -->-encodes any non-ASCII
- * characters, spaces, and non-printing characters in @part.
+ * @unescape_extra) in @part.
  *
  * "Unreserved" characters are those that are not allowed to be used
  * for punctuation according to the URI spec. For example, letters are
@@ -902,8 +725,6 @@ uri_normalized_copy (const char *part, int length,
 char *
 soup_uri_normalize (const char *part, const char *unescape_extra)
 {
-	g_return_val_if_fail (part != NULL, NULL);
-
 	return uri_normalized_copy (part, strlen (part), unescape_extra);
 }
 
@@ -913,7 +734,7 @@ soup_uri_normalize (const char *part, const char *unescape_extra)
  * @uri: a #SoupURI
  *
  * Tests if @uri uses the default port for its scheme. (Eg, 80 for
- * http.) (This only works for http, https and ftp; libsoup does not know
+ * http.) (This only works for http and https; libsoup does not know
  * the default ports of other protocols.)
  *
  * Return value: %TRUE or %FALSE
@@ -921,12 +742,27 @@ soup_uri_normalize (const char *part, const char *unescape_extra)
 gboolean
 soup_uri_uses_default_port (SoupURI *uri)
 {
-	g_return_val_if_fail (uri != NULL, FALSE);
-	g_warn_if_fail (SOUP_URI_IS_VALID (uri));
+	g_return_val_if_fail (uri->scheme == SOUP_URI_SCHEME_HTTP ||
+			      uri->scheme == SOUP_URI_SCHEME_HTTPS ||
+			      uri->scheme == SOUP_URI_SCHEME_FTP, FALSE);
 
 	return uri->port == soup_scheme_default_port (uri->scheme);
 }
 
+/**
+ * SOUP_URI_SCHEME_HTTP:
+ *
+ * "http" as an interned string. This can be compared directly against
+ * the value of a #SoupURI's <structfield>scheme</structfield>
+ **/
+
+/**
+ * SOUP_URI_SCHEME_HTTPS:
+ *
+ * "https" as an interned string. This can be compared directly
+ * against the value of a #SoupURI's <structfield>scheme</structfield>
+ **/
+
 /**
  * soup_uri_get_scheme:
  * @uri: a #SoupURI
@@ -940,8 +776,6 @@ soup_uri_uses_default_port (SoupURI *uri)
 const char *
 soup_uri_get_scheme (SoupURI *uri)
 {
-	g_return_val_if_fail (uri != NULL, NULL);
-
 	return uri->scheme;
 }
 
@@ -956,9 +790,6 @@ soup_uri_get_scheme (SoupURI *uri)
 void
 soup_uri_set_scheme (SoupURI *uri, const char *scheme)
 {
-	g_return_if_fail (uri != NULL);
-	g_return_if_fail (scheme != NULL);
-
 	uri->scheme = soup_uri_parse_scheme (scheme, strlen (scheme));
 	uri->port = soup_scheme_default_port (uri->scheme);
 }
@@ -976,23 +807,19 @@ soup_uri_set_scheme (SoupURI *uri, const char *scheme)
 const char *
 soup_uri_get_user (SoupURI *uri)
 {
-	g_return_val_if_fail (uri != NULL, NULL);
-
 	return uri->user;
 }
 
 /**
  * soup_uri_set_user:
  * @uri: a #SoupURI
- * @user: (allow-none): the username, or %NULL
+ * @user: the username, or %NULL
  *
  * Sets @uri's user to @user.
  **/
 void
 soup_uri_set_user (SoupURI *uri, const char *user)
 {
-	g_return_if_fail (uri != NULL);
-
 	g_free (uri->user);
 	uri->user = g_strdup (user);
 }
@@ -1010,23 +837,19 @@ soup_uri_set_user (SoupURI *uri, const char *user)
 const char *
 soup_uri_get_password (SoupURI *uri)
 {
-	g_return_val_if_fail (uri != NULL, NULL);
-
 	return uri->password;
 }
 
 /**
  * soup_uri_set_password:
  * @uri: a #SoupURI
- * @password: (allow-none): the password, or %NULL
+ * @password: the password, or %NULL
  *
  * Sets @uri's password to @password.
  **/
 void
 soup_uri_set_password (SoupURI *uri, const char *password)
 {
-	g_return_if_fail (uri != NULL);
-
 	g_free (uri->password);
 	uri->password = g_strdup (password);
 }
@@ -1044,29 +867,23 @@ soup_uri_set_password (SoupURI *uri, const char *password)
 const char *
 soup_uri_get_host (SoupURI *uri)
 {
-	g_return_val_if_fail (uri != NULL, NULL);
-
 	return uri->host;
 }
 
 /**
  * soup_uri_set_host:
  * @uri: a #SoupURI
- * @host: (allow-none): the hostname or IP address, or %NULL
+ * @host: the hostname or IP address, or %NULL
  *
  * Sets @uri's host to @host.
  *
  * If @host is an IPv6 IP address, it should not include the brackets
  * required by the URI syntax; they will be added automatically when
  * converting @uri to a string.
- *
- * http and https URIs should not have a %NULL @host.
  **/
 void
 soup_uri_set_host (SoupURI *uri, const char *host)
 {
-	g_return_if_fail (uri != NULL);
-
 	g_free (uri->host);
 	uri->host = g_strdup (host);
 }
@@ -1084,8 +901,6 @@ soup_uri_set_host (SoupURI *uri, const char *host)
 guint
 soup_uri_get_port (SoupURI *uri)
 {
-	g_return_val_if_fail (uri != NULL, 0);
-
 	return uri->port;
 }
 
@@ -1100,8 +915,6 @@ soup_uri_get_port (SoupURI *uri)
 void
 soup_uri_set_port (SoupURI *uri, guint port)
 {
-	g_return_if_fail (uri != NULL);
-
 	uri->port = port;
 }
 
@@ -1118,29 +931,19 @@ soup_uri_set_port (SoupURI *uri, guint port)
 const char *
 soup_uri_get_path (SoupURI *uri)
 {
-	g_return_val_if_fail (uri != NULL, NULL);
-
 	return uri->path;
 }
 
 /**
  * soup_uri_set_path:
  * @uri: a #SoupURI
- * @path: the non-%NULL path
+ * @path: the path
  *
  * Sets @uri's path to @path.
  **/
 void
 soup_uri_set_path (SoupURI *uri, const char *path)
 {
-	g_return_if_fail (uri != NULL);
-
-	/* We allow a NULL path for compatibility, but warn about it. */
-	if (!path) {
-		g_warn_if_fail (path != NULL);
-		path = "";
-	}
-
 	g_free (uri->path);
 	uri->path = g_strdup (path);
 }
@@ -1158,23 +961,19 @@ soup_uri_set_path (SoupURI *uri, const char *path)
 const char *
 soup_uri_get_query (SoupURI *uri)
 {
-	g_return_val_if_fail (uri != NULL, NULL);
-
 	return uri->query;
 }
 
 /**
  * soup_uri_set_query:
  * @uri: a #SoupURI
- * @query: (allow-none): the query
+ * @query: the query
  *
  * Sets @uri's query to @query.
  **/
 void
 soup_uri_set_query (SoupURI *uri, const char *query)
 {
-	g_return_if_fail (uri != NULL);
-
 	g_free (uri->query);
 	uri->query = g_strdup (query);
 }
@@ -1191,8 +990,6 @@ soup_uri_set_query (SoupURI *uri, const char *query)
 void
 soup_uri_set_query_from_form (SoupURI *uri, GHashTable *form)
 {
-	g_return_if_fail (uri != NULL);
-
 	g_free (uri->query);
 	uri->query = soup_form_encode_hash (form);
 }
@@ -1215,8 +1012,6 @@ soup_uri_set_query_from_fields (SoupURI    *uri,
 {
 	va_list args;
 
-	g_return_if_fail (uri != NULL);
-
 	g_free (uri->query);
 	va_start (args, first_field);
 	uri->query = soup_form_encode_valist (first_field, args);
@@ -1236,36 +1031,32 @@ soup_uri_set_query_from_fields (SoupURI    *uri,
 const char *
 soup_uri_get_fragment (SoupURI *uri)
 {
-	g_return_val_if_fail (uri != NULL, NULL);
-
 	return uri->fragment;
 }
 
 /**
  * soup_uri_set_fragment:
  * @uri: a #SoupURI
- * @fragment: (allow-none): the fragment
+ * @fragment: the fragment
  *
  * Sets @uri's fragment to @fragment.
  **/
 void
 soup_uri_set_fragment (SoupURI *uri, const char *fragment)
 {
-	g_return_if_fail (uri != NULL);
-
 	g_free (uri->fragment);
 	uri->fragment = g_strdup (fragment);
 }
 
 /**
  * soup_uri_copy_host:
- * @uri: a #SoupURI
+ * @uri: a #SoupUri
  *
  * Makes a copy of @uri, considering only the protocol, host, and port
  *
- * Return value: the new #SoupURI
+ * Return value: the new #SoupUri
  *
- * Since: 2.28
+ * Since: 2.26.3
  **/
 SoupURI *
 soup_uri_copy_host (SoupURI *uri)
@@ -1273,26 +1064,27 @@ soup_uri_copy_host (SoupURI *uri)
 	SoupURI *dup;
 
 	g_return_val_if_fail (uri != NULL, NULL);
-	g_warn_if_fail (SOUP_URI_IS_VALID (uri));
 
 	dup = soup_uri_new (NULL);
 	dup->scheme = uri->scheme;
 	dup->host   = g_strdup (uri->host);
 	dup->port   = uri->port;
-	dup->path   = g_strdup ("");
+	if (dup->scheme == SOUP_URI_SCHEME_HTTP ||
+	    dup->scheme == SOUP_URI_SCHEME_HTTPS)
+		dup->path = g_strdup ("");
 
 	return dup;
 }
 
 /**
  * soup_uri_host_hash:
- * @key: (type Soup.URI): a #SoupURI with a non-%NULL @host member
+ * @key: (type Soup.URI): a #SoupURI
  *
  * Hashes @key, considering only the scheme, host, and port.
  *
  * Return value: a hash
  *
- * Since: 2.28
+ * Since: 2.26.3
  **/
 guint
 soup_uri_host_hash (gconstpointer key)
@@ -1300,7 +1092,6 @@ soup_uri_host_hash (gconstpointer key)
 	const SoupURI *uri = key;
 
 	g_return_val_if_fail (uri != NULL && uri->host != NULL, 0);
-	g_warn_if_fail (SOUP_URI_IS_VALID (uri));
 
 	return GPOINTER_TO_UINT (uri->scheme) + uri->port +
 		soup_str_case_hash (uri->host);
@@ -1308,15 +1099,15 @@ soup_uri_host_hash (gconstpointer key)
 
 /**
  * soup_uri_host_equal:
- * @v1: (type Soup.URI): a #SoupURI with a non-%NULL @host member
- * @v2: (type Soup.URI): a #SoupURI with a non-%NULL @host member
+ * @v1: (type Soup.URI): a #SoupURI
+ * @v2: (type Soup.URI): a #SoupURI
  *
  * Compares @v1 and @v2, considering only the scheme, host, and port.
  *
  * Return value: whether or not the URIs are equal in scheme, host,
  * and port.
  *
- * Since: 2.28
+ * Since: 2.26.3
  **/
 gboolean
 soup_uri_host_equal (gconstpointer v1, gconstpointer v2)
@@ -1326,8 +1117,6 @@ soup_uri_host_equal (gconstpointer v1, gconstpointer v2)
 
 	g_return_val_if_fail (one != NULL && two != NULL, one == two);
 	g_return_val_if_fail (one->host != NULL && two->host != NULL, one->host == two->host);
-	g_warn_if_fail (SOUP_URI_IS_VALID (one));
-	g_warn_if_fail (SOUP_URI_IS_VALID (two));
 
 	if (one->scheme != two->scheme)
 		return FALSE;
@@ -1337,51 +1126,18 @@ soup_uri_host_equal (gconstpointer v1, gconstpointer v2)
 	return g_ascii_strcasecmp (one->host, two->host) == 0;
 }
 
-gboolean
-soup_uri_is_http (SoupURI *uri, char **aliases)
+
+GType
+soup_uri_get_type (void)
 {
-	int i;
+	static volatile gsize type_volatile = 0;
 
-	if (uri->scheme == SOUP_URI_SCHEME_HTTP ||
-	    uri->scheme == SOUP_URI_SCHEME_WS)
-		return TRUE;
-	else if (uri->scheme == SOUP_URI_SCHEME_HTTPS ||
-		 uri->scheme == SOUP_URI_SCHEME_WSS)
-		return FALSE;
-	else if (!aliases)
-		return FALSE;
-
-	for (i = 0; aliases[i]; i++) {
-		if (uri->scheme == aliases[i])
-			return TRUE;
+	if (g_once_init_enter (&type_volatile)) {
+		GType type = g_boxed_type_register_static (
+			g_intern_static_string ("SoupURI"),
+			(GBoxedCopyFunc) soup_uri_copy,
+			(GBoxedFreeFunc) soup_uri_free);
+		g_once_init_leave (&type_volatile, type);
 	}
-
-	if (aliases[0] && !aliases[1] && !strcmp (aliases[0], "*"))
-		return TRUE;
-	else
-		return FALSE;
+	return type_volatile;
 }
-
-gboolean
-soup_uri_is_https (SoupURI *uri, char **aliases)
-{
-	int i;
-
-	if (uri->scheme == SOUP_URI_SCHEME_HTTPS ||
-	    uri->scheme == SOUP_URI_SCHEME_WSS)
-		return TRUE;
-	else if (uri->scheme == SOUP_URI_SCHEME_HTTP ||
-		 uri->scheme == SOUP_URI_SCHEME_WS)
-		return FALSE;
-	else if (!aliases)
-		return FALSE;
-
-	for (i = 0; aliases[i]; i++) {
-		if (uri->scheme == aliases[i])
-			return TRUE;
-	}
-
-	return FALSE;
-}
-
-G_DEFINE_BOXED_TYPE (SoupURI, soup_uri, soup_uri_copy, soup_uri_free)
